/*
 * T-Plan Robot Enterprise, automated testing tool.
 * Copyright (C) 2009-2012 T-Plan Limited (http://www.t-plan.com),
 * Cloisters House, 8 Battersea Park Road, London, SW8 4BG, United Kingdom.
 * All rights reserved.
 */
package com.tplan.adb;

import com.tplan.robot.util.InputStreamDrain;
import java.io.IOException;
import java.util.List;

/**
 * A thread running an OS process. It allows to stop the process externally
 * through {@link Thread#interrupt()}. Available since version 2.
 * @since 2.0
 */
class OSProcessThread extends Thread {

    Process process;
    StringBuilder stdOut;
    StringBuilder errOut;
    int exitValue = -1;

    OSProcessThread(List<String> cmd, StringBuilder stdOut, StringBuilder errOut) throws IOException {
        this.stdOut = stdOut;
        this.errOut = errOut;
        String cmdArray[] = cmd.toArray(new String[cmd.size()]);
        this.process = Runtime.getRuntime().exec(cmdArray);
    }

    @Override
    public void run() {
        try {
            InputStreamDrain errDrain = new InputStreamDrain(process.getErrorStream());
            InputStreamDrain stdDrain = new InputStreamDrain(process.getInputStream());

            errDrain.start();
            stdDrain.start();
            exitValue = process.waitFor();
            
            // Save variable values to the context
            while (errDrain.isAlive() || stdDrain.isAlive()) {
                try {
                    Thread.sleep(1);
                } catch (InterruptedException e) {
                }
            }
            String s = stdDrain.getOutput();
            if (s != null && !s.isEmpty()) {
                stdOut.append(s);
            }
            s = errDrain.getOutput();
            if (s != null && !s.isEmpty()) {
                errOut.append(s);
            }
        } catch (InterruptedException ex) {
            process.destroy();
        }
    }
}
