/*
 * T-Plan Robot Enterprise, automated testing tool.
 * Copyright (C) 2009-2013 T-Plan Limited (http://www.t-plan.com),
 * Cloisters House, 8 Battersea Park Road, London, SW8 4BG, United Kingdom.
 * All rights reserved.
 */
package com.tplan.adb;

import com.android.ddmlib.CollectingOutputReceiver;
import com.android.ddmlib.IDevice;
import com.tplan.robot.ApplicationSupport;
import com.tplan.robot.util.Utils;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.swing.DefaultListModel;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

/**
 * A dialog allowing to select an application package from the list of all
 * applications loaded from the Android device over the ADB.
 */
class PackageDialog extends JDialog implements Runnable, ActionListener, ListSelectionListener {

    IDevice device;
    ListModel packages;
    JList list = new JList();
    JButton btnOk = new JButton("Choose", ApplicationSupport.getImageIcon("ok15.gif"));
    private String selectedPackage = null;
    private String packageToSelect = null;
    Component owningButton;

    PackageDialog(Component owningButton, IDevice device) {
        super(SwingUtilities.getWindowAncestor(owningButton), ModalityType.APPLICATION_MODAL);
        this.owningButton = owningButton;
        String name = device == null ? "Package Viewer" : device.getName() + " Packages";
        setTitle(name);
        this.device = device;
        JPanel pnl = new JPanel(new BorderLayout());
        pnl.add(new JScrollPane(list), BorderLayout.CENTER);
        JPanel south = new JPanel(new FlowLayout(FlowLayout.CENTER));
        btnOk.setEnabled(false);
        btnOk.setActionCommand("ok");
        btnOk.addActionListener(this);
        south.add(btnOk);
        JButton btnFromFile = new JButton("Read from .apk", ApplicationSupport.getImageIcon("open16.gif"));
        btnFromFile.setActionCommand("file");
        btnFromFile.addActionListener(this);
        south.add(btnFromFile);
        JButton btnCancel = new JButton("Cancel", ApplicationSupport.getImageIcon("cancel16.png"));
        btnCancel.setActionCommand("cancel");
        btnCancel.addActionListener(this);
        south.add(btnCancel);
        pnl.add(south, BorderLayout.SOUTH);
        setContentPane(pnl);
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        Utils.registerDialogForEscape(this, btnCancel);
        getRootPane().setDefaultButton(btnOk);

        list.setVisibleRowCount(12);
        list.setEnabled(false);
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        DefaultListModel model = new DefaultListModel();
        if (device != null) {
            model.add(0, "Wait while retrieving packages...");
        } else {
            model.add(0, "No device is currently connected.");
        }
        list.setModel(model);
        list.addListSelectionListener(this);
    }

    @Override
    public void setVisible(boolean visible) {
        if (visible) {
            packages = null;
            pack();
            Dimension d = getSize();
            d.width = Math.max(d.width, 420);
            setSize(d);
            setLocationRelativeTo(getParent());
            if (device != null) {
                new Thread(this).start();
            }
        }
        super.setVisible(visible);
    }

    public void run() {
        if (packages == null) {
            CollectingOutputReceiver receiver = new CollectingOutputReceiver();
            DefaultListModel m = new DefaultListModel();
            packages = m;
            try {
                device.executeShellCommand("pm list packages", receiver);
                BufferedReader reader = new BufferedReader(new StringReader(receiver.getOutput()));
                String line;
                List<String> l = new ArrayList<String>();
                while ((line = reader.readLine()) != null) {
                    if (line.startsWith("package:")) {
                        line = line.substring("package:".length());
                    }
                    l.add(line);
                }
                Collections.sort(l);
                for (String s : l) {
                    m.add(m.getSize(), s);
                }
            } catch (Exception ex) {
                Utils.showErrorDialog(this, "Failed To Retrieve Packages",
                        "Failed to retrieve packages from device " + device.getName(), ex, true);
            }
            if (isVisible()) {  // May have been canceled
                SwingUtilities.invokeLater(this);
            }
        } else {
            list.setEnabled(true);
            list.setModel(packages);
            if (packages.getSize() > 0) {
                int index = 0;
                if (packageToSelect != null) {
                    for (int i = 0; i < packages.getSize(); i++) {
                        if (packageToSelect.equals(packages.getElementAt(i))) {
                            index = i;
                            break;
                        }
                    }
                }
                list.setSelectedIndex(index);
                Rectangle r = list.getUI().getCellBounds(list, index, index);
                if (r != null) {
                    r.height += r.height;
                    list.scrollRectToVisible(r);
                }
            }
        }
    }

    public void actionPerformed(ActionEvent e) {
        if ("ok".equals(e.getActionCommand())) {
            selectedPackage = list.getSelectedValue().toString();
            dispose();
        } else if ("cancel".equals(e.getActionCommand())) {
            selectedPackage = null;
            dispose();
        } else if ("file".equals(e.getActionCommand())) {
            JFileChooser chooser = new JFileChooser();
            if (Install.lastAppFile != null) {
                chooser.setCurrentDirectory(Install.lastAppFile.getParentFile());
                chooser.setSelectedFile(Install.lastAppFile);
            }
            chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
            FileFilter filter = new FileNameExtensionFilter("Android applications (*.apk)", "apk");
            chooser.setFileFilter(filter);
            chooser.setAcceptAllFileFilterUsed(false);
            if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
                File apk = chooser.getSelectedFile();
                Install.lastAppFile = apk;
                try {
                    // v2 update - the method returns an array of [package, activity]
                    String s[] = Install.readPackage(apk.getAbsolutePath());
                    String pkg = s[0];
                    if (pkg == null) {
                        throw new IOException("Unknown .apk format! Please submit the app to the T-Plan support for review.");
                    }
                    String options[] = new String[]{"Choose & Close", "Cancel"};
                    Icon icons[] = new Icon[]{
                        ApplicationSupport.getImageIcon("ok15.gif"),
                        ApplicationSupport.getImageIcon("cancel16.png")
                    };
                    // v2 update - the method returns an array of [package, activity]
                    int opt = Utils.showMessageDialog(this, "Package From APK",
                            "<html><body>The " + apk.getName() + " file contains an application:<br/><br/><ul>"
                            + "Package:  <code>" + pkg + "</code><br/>"
                            + "Activity: <code>" + s[1] + "</code>"
                            + "</ul></body></html>",
                            null, options, icons, options[0]);
                    if (opt == 0) {
                        selectedPackage = pkg;
                        dispose();
                    }
                } catch (IOException ex) {
                    Utils.showErrorDialog(this, "I/O Error", "Failed to retrieve the package from " + apk.getName() + ":\n" + ex.getMessage(), ex, true);
                }
            }
        }
    }

    public void valueChanged(ListSelectionEvent e) {
        btnOk.setEnabled(device != null && list.isEnabled() && e.getFirstIndex() >= 0);
    }

    /**
     * Get the selected package.
     *
     * @return the package selected in the window or null if the window was
     * canceled.
     */
    public String getSelectedPackage() {
        return selectedPackage;
    }

    /**
     * Set the package to select when the window gets opened.
     *
     * @param packageToSelect the package to select.
     */
    public void setSelectedPackage(String packageToSelect) {
        this.packageToSelect = packageToSelect;
    }
}
