/*
 * T-Plan Robot Enterprise, automated testing tool.
 * Copyright (C) 2009-2013 T-Plan Limited (http://www.t-plan.com),
 * Cloisters House, 8 Battersea Park Road, London, SW8 4BG, United Kingdom.
 * All rights reserved.
 */
package com.tplan.adb;

import com.android.ddmlib.IDevice;
import com.tplan.robot.gui.MainFrame;
import com.tplan.robot.gui.components.StringComponent;
import com.tplan.robot.preferences.Preference;
import com.tplan.robot.preferences.PreferenceComponentManager;
import com.tplan.robot.remoteclient.RemoteDesktopClient;
import com.tplan.robot.remoteclient.adb.AdbClient;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Class allowing to bind the application package field in the Properties 
 * window of a script with the window displaying the list of packages installed 
 * on the connected device.
 * @since 2.0
 */
class PackageListManager implements PreferenceComponentManager, ActionListener {
    
    private StringComponent packageComponent;

    public void componentCreated(Preference p, Object component, Object container) {
        if (component instanceof StringComponent) {
            packageComponent = (StringComponent) component;
            StringComponent sc = (StringComponent) component;
            sc.getButton().addActionListener(this);
        }
    }

    public void goingToLoad(Preference p, Object component, Object container, Object configuration) {
    }

    public void savePerformed(Preference p, Object component, Object container, Object configuration) {
    }

    public void actionPerformed(ActionEvent e) {
        IDevice dev = null;
        RemoteDesktopClient client = MainFrame.getInstance().getClient();
        if (client != null && client.isConnected() && client instanceof AdbClient) {
            AdbClient cl = (AdbClient) client;
            dev = (IDevice) cl.getIDevice();
        }
        Component owner = (Component) e.getSource();
        PackageDialog dlg = new PackageDialog(owner, dev);
        String selected = packageComponent.getText();
        if (selected != null && !selected.isEmpty()) {
            dlg.setSelectedPackage(selected);
        }
        dlg.setVisible(true);
        String s = dlg.getSelectedPackage();
        if (s != null) {
            packageComponent.setText(s);
        }
    }
}
